! ----------------------------------------------- !
!                   IXHANDLE                      !
!  Gestion des handle (références sur objets C++) !
! ----------------------------------------------- !
MODULE IXHANDLE
   USE ISO_C_BINDING

   ! Ne jamais modifier les valeurs internes de ce type dans le code fortran.
   TYPE XHandle
      INTEGER(kind = C_INT) :: dtype
      INTEGER(kind = C_INT) :: daddr
   END TYPE XHandle

   ! enum XDType
   INTEGER(kind = C_INT), PARAMETER  :: NOTYPE = 0
   INTEGER(kind = C_INT), PARAMETER  :: DTREATMENT = 1, DDATE = 2, CALENDAR = 3, ECONTEXT = 4
   INTEGER(kind = C_INT), PARAMETER  :: EAXIS = 5 , EDOMAIN = 6 , EFIELD = 7 , EFILE = 8 , EGRID = 9
   INTEGER(kind = C_INT), PARAMETER  :: GAXIS = 10, GDOMAIN = 11, GFIELD = 12, GFILE = 13, GGRID = 14

   ! Autres constantes
   TYPE(XHandle)  , PARAMETER :: NULLHANDLE = XHandle(0, 0)

   INTERFACE ! Ne pas appeler directement/Interface FORTRAN 2003 <-> C99

      SUBROUTINE xios_handle_create(ret, dtype, idt, idt_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT)                :: ret
         INTEGER  (kind = C_INT), VALUE         :: dtype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: idt
         INTEGER  (kind = C_INT), VALUE         :: idt_size
      END SUBROUTINE xios_handle_create

   END INTERFACE

   CONTAINS

   ! Permet de créer un handle sur un objet c++ à partir d'un identifiant (dépend du contexte).
   SUBROUTINE handle_create(ret, dtype, idt)
         TYPE(XHandle), INTENT(OUT)     :: ret
         INTEGER, INTENT(IN)            :: dtype
         CHARACTER(len = *), INTENT(IN) :: idt
      CALL xios_handle_create(ret%daddr, dtype, idt, len(idt))

      IF (ret%daddr == 0) THEN
         PRINT *, "(F2003 interface) Impossible de créer un handle sur l'élément !"
         STOP
      ELSE
         ret%dtype = dtype
      END IF
   END SUBROUTINE handle_create

END MODULE IXHANDLE

! --------------------------------------------- !
!                   ICALENDAR                   !
!        Prise en charge des calendriers        !
! --------------------------------------------- !

MODULE ICALENDAR
   USE ISO_C_BINDING

   ! enum XCalendarType
   INTEGER(kind = C_INT), PARAMETER  :: D360 = 0 , ALLLEAP = 1 , NOLEAP = 2 , JULIAN = 3 , GREGORIAN = 4

   TYPE XDate
      INTEGER :: year, month, day, hour, minute, second
   END TYPE XDate

   TYPE XDuration
      REAL(kind = 8) :: year, month, day, hour, minute, second
   END TYPE XDuration

END MODULE ICALENDAR

! --------------------------------------------- !
!                    IXMLTREE                   !
!         Modification des document XML         !
! --------------------------------------------- !
MODULE IXMLTREE
   USE ISO_C_BINDING

   INTERFACE ! Ne pas appeler directement/Interface FORTRAN 2003 <-> C99

      SUBROUTINE xios_xml_tree_add(parent_, parent_type, child_, child_type, child_id, child_id_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: parent_
         INTEGER  (kind = C_INT), VALUE         :: parent_type
         INTEGER  (kind = C_INT)                :: child_
         INTEGER  (kind = C_INT), VALUE         :: child_type
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: child_id
         INTEGER  (kind = C_INT), VALUE         :: child_id_size
      END SUBROUTINE xios_xml_tree_add

      SUBROUTINE  xios_xml_tree_show(filename, filename_size) bind(C)
         import C_CHAR, C_INT
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: filename
         INTEGER  (kind = C_INT), VALUE         :: filename_size
      END SUBROUTINE xios_xml_tree_show

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Attribut des éléments de type field et field_group !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE  xios_xml_set_field_name(field, ftype, fname, fname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fname
         INTEGER  (kind = C_INT), VALUE         :: fname_size
      END SUBROUTINE xios_xml_set_field_name

      SUBROUTINE  xios_xml_set_field_sname(field, ftype, fsname, fsname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fsname
         INTEGER  (kind = C_INT), VALUE         :: fsname_size
      END SUBROUTINE xios_xml_set_field_sname

      SUBROUTINE  xios_xml_set_field_lname(field, ftype, flname, flname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: flname
         INTEGER  (kind = C_INT), VALUE         :: flname_size
      END SUBROUTINE xios_xml_set_field_lname

      SUBROUTINE  xios_xml_set_field_unit(field, ftype, funit, funit_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: funit
         INTEGER  (kind = C_INT), VALUE         :: funit_size
      END SUBROUTINE xios_xml_set_field_unit

      SUBROUTINE  xios_xml_set_field_operation(field, ftype, foperation, foperation_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: foperation
         INTEGER  (kind = C_INT), VALUE         :: foperation_size
      END SUBROUTINE xios_xml_set_field_operation

      SUBROUTINE  xios_xml_set_field_freq_op(field, ftype, year, month, day, hour, minute, second) bind(C)
         import C_DOUBLE, C_INT
         INTEGER (kind = C_INT), VALUE  :: field, ftype
         REAL (kind = C_DOUBLE), VALUE  :: year, month, day, hour, minute, second
      END SUBROUTINE xios_xml_set_field_freq_op

      SUBROUTINE  xios_xml_set_field_level(field, ftype, flevel) bind(C)
         import C_INT
         INTEGER (kind = C_INT), VALUE  :: field, ftype
         INTEGER (kind = C_INT), VALUE  :: flevel
      END SUBROUTINE xios_xml_set_field_level

      SUBROUTINE  xios_xml_set_field_prec(field, ftype, fprec) bind(C)
         import C_INT
         INTEGER (kind = C_INT), VALUE  :: field, ftype
         INTEGER (kind = C_INT), VALUE  :: fprec
      END SUBROUTINE xios_xml_set_field_prec

      SUBROUTINE  xios_xml_set_field_enabled(field, ftype, fenabled) bind(C)
         import C_INT, C_BOOL
         INTEGER (kind = C_INT), VALUE  :: field, ftype
         LOGICAL (kind = C_BOOL), VALUE :: fenabled
      END SUBROUTINE xios_xml_set_field_enabled

      SUBROUTINE  xios_xml_set_field_dref(field, ftype, fdref, fdref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fdref
         INTEGER  (kind = C_INT), VALUE         :: fdref_size
      END SUBROUTINE xios_xml_set_field_dref

      SUBROUTINE  xios_xml_set_field_aref(field, ftype, faref, faref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: faref
         INTEGER  (kind = C_INT), VALUE         :: faref_size
      END SUBROUTINE xios_xml_set_field_aref

      SUBROUTINE  xios_xml_set_field_gref(field, ftype, fgref, fgref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fgref
         INTEGER  (kind = C_INT), VALUE         :: fgref_size
      END SUBROUTINE xios_xml_set_field_gref

      SUBROUTINE  xios_xml_set_field_zref(field, ftype, fzref, fzref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fzref
         INTEGER  (kind = C_INT), VALUE         :: fzref_size
      END SUBROUTINE xios_xml_set_field_zref

      SUBROUTINE  xios_xml_set_field_fref(field, ftype, ffref, ffref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: field, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: ffref
         INTEGER  (kind = C_INT), VALUE         :: ffref_size
      END SUBROUTINE xios_xml_set_field_fref

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Attribut des éléments de type file et file_group !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      SUBROUTINE  xios_xml_set_file_name(file_, ftype, fname, fname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: file_, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fname
         INTEGER  (kind = C_INT), VALUE         :: fname_size
      END SUBROUTINE xios_xml_set_file_name

      SUBROUTINE  xios_xml_set_file_description(file_, ftype, fdescription, fdescription_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: file_, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: fdescription
         INTEGER  (kind = C_INT), VALUE         :: fdescription_size
      END SUBROUTINE xios_xml_set_file_description

      SUBROUTINE  xios_xml_set_file_output_freq(file_, ftype, year, month, day, hour, minute, second) bind(C)
         import C_DOUBLE, C_INT
         INTEGER (kind = C_INT), VALUE :: file_, ftype
         REAL (kind = C_DOUBLE), VALUE :: year, month, day, hour, minute, second
      END SUBROUTINE xios_xml_set_file_output_freq

      SUBROUTINE  xios_xml_set_file_olevel(file_, ftype, folevel) bind(C)
         import C_INT
         INTEGER (kind = C_INT), VALUE :: file_, ftype
         INTEGER (kind = C_INT), VALUE :: folevel
      END SUBROUTINE xios_xml_set_file_olevel

      SUBROUTINE  xios_xml_set_file_enabled(file_, ftype, fenabled) bind(C)
         import C_INT, C_BOOL
         INTEGER (kind = C_INT), VALUE  :: file_, ftype
         LOGICAL (kind = C_BOOL), VALUE :: fenabled
      END SUBROUTINE xios_xml_set_file_enabled

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Attribut des éléments de type grid et grid_group !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE  xios_xml_set_grid_name(grid, ftype, gname, gname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: grid, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: gname
         INTEGER  (kind = C_INT), VALUE         :: gname_size
      END SUBROUTINE xios_xml_set_grid_name

      SUBROUTINE  xios_xml_set_grid_description(grid, ftype, gdescription, gdescription_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: grid, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: gdescription
         INTEGER  (kind = C_INT), VALUE         :: gdescription_size
      END SUBROUTINE xios_xml_set_grid_description

      SUBROUTINE  xios_xml_set_grid_dref(grid, ftype, dref, dref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: grid, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: dref
         INTEGER  (kind = C_INT), VALUE         :: dref_size
      END SUBROUTINE xios_xml_set_grid_dref

      SUBROUTINE  xios_xml_set_grid_aref(grid, ftype, aref, aref_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: grid, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: aref
         INTEGER  (kind = C_INT), VALUE         :: aref_size
      END SUBROUTINE xios_xml_set_grid_aref

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Attribut des éléments de type axis et axis_group !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      SUBROUTINE  xios_xml_set_axis_name(axis, ftype, aname, aname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: axis, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: aname
         INTEGER  (kind = C_INT), VALUE         :: aname_size
      END SUBROUTINE xios_xml_set_axis_name

      SUBROUTINE  xios_xml_set_axis_sname(axis, ftype, asname, asname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: axis, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: asname
         INTEGER  (kind = C_INT), VALUE         :: asname_size
      END SUBROUTINE xios_xml_set_axis_sname

      SUBROUTINE  xios_xml_set_axis_lname(axis, ftype, alname, alname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: axis, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: alname
         INTEGER  (kind = C_INT), VALUE         :: alname_size
      END SUBROUTINE xios_xml_set_axis_lname

      SUBROUTINE  xios_xml_set_axis_unit(axis, ftype, aunit, aunit_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: axis, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: aunit
         INTEGER  (kind = C_INT), VALUE         :: aunit_size
      END SUBROUTINE xios_xml_set_axis_unit

      SUBROUTINE  xios_xml_set_axis_value(axis, ftype, avalue, avalue_size) bind(C)
         import C_DOUBLE, C_INT
         INTEGER  (kind = C_INT), VALUE      :: axis, ftype
         REAL(kind = C_DOUBLE), DIMENSION(*) :: avalue
         INTEGER  (kind = C_INT), VALUE      :: avalue_size
      END SUBROUTINE xios_xml_set_axis_value

      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Attribut des éléments de type domain et domain_group !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      SUBROUTINE  xios_xml_set_domain_name(domain, ftype, dname, dname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: domain, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: dname
         INTEGER  (kind = C_INT), VALUE         :: dname_size
      END SUBROUTINE xios_xml_set_domain_name

      SUBROUTINE  xios_xml_set_domain_sname(domain, ftype, dsname, dsname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: domain, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: dsname
         INTEGER  (kind = C_INT), VALUE         :: dsname_size
      END SUBROUTINE xios_xml_set_domain_sname

      SUBROUTINE  xios_xml_set_domain_lname(domain, ftype, dlname, dlname_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: domain, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: dlname
         INTEGER  (kind = C_INT), VALUE         :: dlname_size
      END SUBROUTINE xios_xml_set_domain_lname

      SUBROUTINE  xios_xml_set_domain_niglo(domain, ftype, niglo) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, niglo
      END SUBROUTINE xios_xml_set_domain_niglo

      SUBROUTINE  xios_xml_set_domain_njglo(domain, ftype, njglo) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, njglo
      END SUBROUTINE xios_xml_set_domain_njglo

      SUBROUTINE  xios_xml_set_domain_ibegin(domain, ftype, ibegin) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, ibegin
      END SUBROUTINE xios_xml_set_domain_ibegin

      SUBROUTINE  xios_xml_set_domain_iend(domain, ftype, iend) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, iend
      END SUBROUTINE xios_xml_set_domain_iend

      SUBROUTINE  xios_xml_set_domain_ni(domain, ftype, ni) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, ni
      END SUBROUTINE xios_xml_set_domain_ni

      SUBROUTINE  xios_xml_set_domain_jbegin(domain, ftype, jbegin) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, jbegin
      END SUBROUTINE xios_xml_set_domain_jbegin

      SUBROUTINE  xios_xml_set_domain_jend(domain, ftype, jend) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, jend
      END SUBROUTINE xios_xml_set_domain_jend

      SUBROUTINE  xios_xml_set_domain_nj(domain, ftype, nj) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, nj
      END SUBROUTINE xios_xml_set_domain_nj

      SUBROUTINE  xios_xml_set_domain_mask(domain, ftype, mask, maskXsize, maskYsize) bind(C)
         import C_BOOL, C_INT
         INTEGER  (kind = C_INT), VALUE       :: domain, ftype
         LOGICAL(kind = C_BOOL), DIMENSION(*) :: mask
         INTEGER  (kind = C_INT), VALUE       :: maskXsize, maskYsize
      END SUBROUTINE xios_xml_set_domain_mask

      SUBROUTINE  xios_xml_set_domain_ddim(domain, ftype, dadim) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, dadim
      END SUBROUTINE xios_xml_set_domain_ddim

      SUBROUTINE  xios_xml_set_domain_dni(domain, ftype, dni) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, dni
      END SUBROUTINE xios_xml_set_domain_dni

      SUBROUTINE  xios_xml_set_domain_dnj(domain, ftype, dnj) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, dnj
      END SUBROUTINE xios_xml_set_domain_dnj

      SUBROUTINE  xios_xml_set_domain_dibegin(domain, ftype, dibegin) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, dibegin
      END SUBROUTINE xios_xml_set_domain_dibegin

      SUBROUTINE  xios_xml_set_domain_djbegin(domain, ftype, djbegin) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, djbegin
      END SUBROUTINE xios_xml_set_domain_djbegin

      SUBROUTINE  xios_xml_set_domain_dnindex(domain, ftype, dnindex) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE :: domain, ftype, dnindex
      END SUBROUTINE xios_xml_set_domain_dnindex

      SUBROUTINE  xios_xml_set_domain_diindex(domain, ftype, diindex, diindex_size) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE      :: domain, ftype
         INTEGER(kind = C_INT), DIMENSION(*) :: diindex
         INTEGER  (kind = C_INT), VALUE      :: diindex_size
      END SUBROUTINE xios_xml_set_domain_diindex

      SUBROUTINE  xios_xml_set_domain_djindex(domain, ftype, djindex, djindex_size) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE      :: domain, ftype
         INTEGER(kind = C_INT), DIMENSION(*) :: djindex
         INTEGER  (kind = C_INT), VALUE      :: djindex_size
      END SUBROUTINE xios_xml_set_domain_djindex

      SUBROUTINE  xios_xml_set_domain_lonvalue(domain, ftype, lonvalue, lonvalue_xsize, lonvalue_ysize) bind(C)
         import C_DOUBLE, C_INT
         INTEGER  (kind = C_INT), VALUE      :: domain, ftype
         REAL(kind = C_DOUBLE), DIMENSION(*) :: lonvalue
         INTEGER  (kind = C_INT), VALUE      :: lonvalue_xsize, lonvalue_ysize
      END SUBROUTINE xios_xml_set_domain_lonvalue

      SUBROUTINE  xios_xml_set_domain_latvalue(domain, ftype, latvalue, latvalue_xsize, lonvalue_ysize) bind(C)
         import C_DOUBLE, C_INT
         INTEGER  (kind = C_INT), VALUE      :: domain, ftype
         REAL(kind = C_DOUBLE), DIMENSION(*) :: latvalue
         INTEGER  (kind = C_INT), VALUE      :: latvalue_xsize, lonvalue_ysize
      END SUBROUTINE xios_xml_set_domain_latvalue

      SUBROUTINE  xios_xml_set_domain_domtype(domain, ftype, domtype, domtype_size) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT), VALUE         :: domain, ftype
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: domtype
         INTEGER  (kind = C_INT), VALUE         :: domtype_size
      END SUBROUTINE xios_xml_set_domain_domtype

   END INTERFACE

   CONTAINS ! Fonctions disponibles pour les utilisateurs.

   SUBROUTINE xml_tree_add(parent_hdl, child_hdl, child_type, child_id)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                :: parent_hdl
      TYPE(XHandle), INTENT(OUT)               :: child_hdl
      INTEGER, INTENT(IN)                      :: child_type
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: child_id
      child_hdl = NULLHANDLE
      IF (((parent_hdl%dtype >= 10) .AND. (parent_hdl%dtype <= 14) .AND.                     &
          (parent_hdl%dtype == child_type) .OR. (parent_hdl%dtype == (child_type + 5))) .OR. &
          ((parent_hdl%dtype == EFILE) .AND. ((child_type == EFIELD) .OR.                    &
          (child_type == GFIELD)))) THEN
         IF (PRESENT(child_id)) THEN
            CALL xios_xml_tree_add(parent_hdl%daddr, parent_hdl%dtype, child_hdl%daddr, child_type, child_id, len(child_id))
         ELSE
            CALL xios_xml_tree_add(parent_hdl%daddr, parent_hdl%dtype, child_hdl%daddr, child_type, "NONE", -1)
         END IF
         IF (child_hdl%daddr == 0) THEN
            PRINT *, "(F2003 interface) Impossible d'ajouter un élément à l'arborescence 0!"
            STOP
         ELSE
            child_hdl%dtype = child_type
         END IF
      ELSE
         PRINT *, "(F2003 interface) Impossible d'ajouter un élément à l'arborescence 1!"
         STOP
      END IF

   END SUBROUTINE xml_tree_add

   SUBROUTINE xml_tree_show(filename)
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: filename
      IF (PRESENT(filename)) THEN
         CALL xios_xml_tree_show(filename, len(filename))
      ELSE
         CALL xios_xml_tree_show("NONE", -1)
      END IF
   END SUBROUTINE xml_tree_show

   SUBROUTINE xml_field_addAttribut(field_hdl, fname, fstandard_name, flong_name, funit, foperation, ffreq_op, &
                                    flevel, fprec, fenabled, fdomain_ref, faxis_ref, fgrid_ref, fzoom_ref, ffield_ref)
      USE IXHANDLE
      USE ICALENDAR
      TYPE(XHandle), INTENT(IN)                :: field_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fname, fstandard_name, flong_name, funit, foperation
      TYPE(XDuration), OPTIONAL, INTENT(IN)    :: ffreq_op
      INTEGER, OPTIONAL, INTENT(IN)            :: flevel, fprec
      LOGICAL (kind = 1), OPTIONAL, INTENT(IN) :: fenabled
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fdomain_ref, faxis_ref, fgrid_ref, fzoom_ref, ffield_ref
      IF ((field_hdl%daddr == 0) .OR. &
          .NOT.((field_hdl%dtype == EFIELD) .OR. (field_hdl%dtype == GFIELD))) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au champ !"
         STOP
      ELSE
         IF (PRESENT(fname))          THEN
            CALL xios_xml_set_field_name(field_hdl%daddr, field_hdl%dtype, fname, len(fname));
         END IF
         IF (PRESENT(fstandard_name)) THEN
            CALL xios_xml_set_field_sname(field_hdl%daddr, field_hdl%dtype, fstandard_name, len(fstandard_name));
         END IF
         IF (PRESENT(flong_name))     THEN
            CALL xios_xml_set_field_lname(field_hdl%daddr, field_hdl%dtype, flong_name, len(flong_name));
         END IF
         IF (PRESENT(funit))          THEN
            CALL xios_xml_set_field_unit(field_hdl%daddr, field_hdl%dtype, funit, len(funit));
         END IF
         IF (PRESENT(foperation))     THEN
            CALL xios_xml_set_field_operation(field_hdl%daddr, field_hdl%dtype, foperation, len(foperation));
         END IF
         IF (PRESENT(ffreq_op))       THEN
            CALL xios_xml_set_field_freq_op(field_hdl%daddr, field_hdl%dtype,            &
                                            ffreq_op%year, ffreq_op%month, ffreq_op%day, &
                                            ffreq_op%hour, ffreq_op%minute, ffreq_op%second)
         END IF
         IF (PRESENT(flevel))         THEN
            CALL xios_xml_set_field_level(field_hdl%daddr, field_hdl%dtype, flevel);
         END IF
         IF (PRESENT(fprec))          THEN
            CALL xios_xml_set_field_prec(field_hdl%daddr, field_hdl%dtype, fprec);
         END IF
         IF (PRESENT(fenabled))       THEN
            CALL xios_xml_set_field_enabled(field_hdl%daddr, field_hdl%dtype, fenabled);
         END IF
         IF (PRESENT(fdomain_ref))    THEN
            CALL xios_xml_set_field_dref(field_hdl%daddr, field_hdl%dtype, fdomain_ref, len(fdomain_ref));
         END IF
         IF (PRESENT(faxis_ref))      THEN
            CALL xios_xml_set_field_aref(field_hdl%daddr, field_hdl%dtype, faxis_ref, len(faxis_ref));
         END IF
         IF (PRESENT(fgrid_ref))      THEN
            CALL xios_xml_set_field_gref(field_hdl%daddr, field_hdl%dtype, fgrid_ref, len(fgrid_ref));
         END IF
         IF (PRESENT(fzoom_ref))      THEN
            CALL xios_xml_set_field_zref(field_hdl%daddr, field_hdl%dtype, fzoom_ref, len(fzoom_ref));
         END IF
         IF (PRESENT(ffield_ref))     THEN
            CALL xios_xml_set_field_fref(field_hdl%daddr, field_hdl%dtype, ffield_ref, len(ffield_ref));
         END IF
      END IF
   END SUBROUTINE xml_field_addAttribut

   SUBROUTINE xml_group_field_addAttribut(gfield_hdl, fname, fstandard_name, flong_name, funit, foperation, ffreq_op, &
                                          flevel, fprec, fenabled, fdomain_ref, faxis_ref, fgrid_ref, fzoom_ref, ffield_ref)
      USE IXHANDLE
      USE ICALENDAR
      TYPE(XHandle), INTENT(IN)                :: gfield_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fname, fstandard_name, flong_name, funit, foperation
      TYPE(XDuration), OPTIONAL, INTENT(IN)    :: ffreq_op
      INTEGER, OPTIONAL, INTENT(IN)            :: flevel, fprec
      LOGICAL (kind = 1), OPTIONAL, INTENT(IN) :: fenabled
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fdomain_ref, faxis_ref, fgrid_ref, fzoom_ref, ffield_ref
      IF ((gfield_hdl%daddr == 0) .OR. .NOT.(gfield_hdl%dtype == GFIELD)) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au groupe de champs !"
         STOP
      ELSE
        CALL xml_field_addAttribut(gfield_hdl, fname, fstandard_name, flong_name, funit, foperation, ffreq_op, &
                                   flevel, fprec, fenabled, fdomain_ref, faxis_ref, fgrid_ref, fzoom_ref, ffield_ref)
      END IF
   END SUBROUTINE xml_group_field_addAttribut

   SUBROUTINE xml_file_addAttribut(file_hdl, fname, fdescription, fofreq, folevel, fenabled)
      USE IXHANDLE
      USE ICALENDAR
      TYPE(XHandle), INTENT(IN)                :: file_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fname, fdescription
      TYPE(XDuration), OPTIONAL, INTENT(IN)    :: fofreq
      INTEGER, OPTIONAL, INTENT(IN)            :: folevel
      LOGICAL (kind = 1), OPTIONAL, INTENT(IN) :: fenabled
      IF ((file_hdl%daddr == 0) .OR. &
          .NOT.((file_hdl%dtype == EFILE) .OR. (file_hdl%dtype == GFILE))) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au fichier !"
         STOP
      ELSE
         IF (PRESENT(fname))        THEN
            CALL xios_xml_set_file_name(file_hdl%daddr, file_hdl%dtype, fname, len(fname));
         END IF
         IF (PRESENT(fdescription)) THEN
            CALL xios_xml_set_file_description(file_hdl%daddr, file_hdl%dtype, fdescription, len(fdescription));
         END IF
         IF (PRESENT(fofreq))     THEN
            CALL xios_xml_set_file_output_freq(file_hdl%daddr, file_hdl%dtype,        &
                                               fofreq%year, fofreq%month, fofreq%day, &
                                               fofreq%hour, fofreq%minute, fofreq%second)
         END IF
         IF (PRESENT(folevel))      THEN
            CALL xios_xml_set_file_olevel(file_hdl%daddr, file_hdl%dtype, folevel);
         END IF
         IF (PRESENT(fenabled))     THEN
            CALL xios_xml_set_file_enabled(file_hdl%daddr, file_hdl%dtype, fenabled);
         END IF
      END IF
   END SUBROUTINE xml_file_addAttribut

   SUBROUTINE xml_group_file_addAttribut(gfile_hdl, fname, fdescription, fofreq, folevel, fenabled)
      USE IXHANDLE
      USE ICALENDAR
      TYPE(XHandle), INTENT(IN)                :: gfile_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: fname, fdescription
      TYPE(XDuration), OPTIONAL, INTENT(IN)    :: fofreq
      INTEGER, OPTIONAL, INTENT(IN)            :: folevel
      LOGICAL (kind = 1), OPTIONAL, INTENT(IN) :: fenabled
      IF ((gfile_hdl%daddr == 0) .OR. .NOT.(gfile_hdl%dtype == GFILE)) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au groupe de fichiers !"
         STOP
      ELSE
        CALL xml_file_addAttribut(gfile_hdl, fname, fdescription, fofreq, folevel, fenabled)
      END IF
   END SUBROUTINE xml_group_file_addAttribut

   SUBROUTINE xml_grid_addAttribut(grid_hdl, gname, gdescription, gdomain_ref, gaxis_ref)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                :: grid_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: gname, gdescription, gdomain_ref, gaxis_ref
      IF ((grid_hdl%daddr == 0) .OR. &
          .NOT.((grid_hdl%dtype == EGRID) .OR. (grid_hdl%dtype == GGRID))) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs à la grille !"
         STOP
      ELSE
         IF (PRESENT(gname))        THEN
            CALL xios_xml_set_grid_name(grid_hdl%daddr, grid_hdl%dtype, gname, len(gname));
         END IF
         IF (PRESENT(gdescription)) THEN
            CALL xios_xml_set_grid_description(grid_hdl%daddr, grid_hdl%dtype, gdescription, len(gdescription));
         END IF
         IF (PRESENT(gdomain_ref))  THEN
            CALL xios_xml_set_grid_dref(grid_hdl%daddr, grid_hdl%dtype, gdomain_ref, len(gdomain_ref));
         END IF
         IF (PRESENT(gaxis_ref))    THEN
            CALL xios_xml_set_grid_aref(grid_hdl%daddr, grid_hdl%dtype, gaxis_ref, len(gaxis_ref));
         END IF
      END IF
   END SUBROUTINE xml_grid_addAttribut

   SUBROUTINE xml_group_grid_addAttribut(ggrid_hdl, gname, gdescription, gdomain_ref, gaxis_ref)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                :: ggrid_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN) :: gname, gdescription, gdomain_ref, gaxis_ref
      IF ((ggrid_hdl%daddr == 0) .OR. .NOT.(ggrid_hdl%dtype == GGRID)) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au groupe de grilles !"
         STOP
      ELSE
        CALL xml_grid_addAttribut(ggrid_hdl, gname, gdescription, gdomain_ref, gaxis_ref)
      END IF
   END SUBROUTINE xml_group_grid_addAttribut

   SUBROUTINE xml_axis_addAttribut(axis_hdl, aname, astandard_name, along_name, aunit, avalue)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                          :: axis_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN)           :: aname, astandard_name, along_name, aunit
      REAL(kind = 8), dimension(*), OPTIONAL, INTENT(IN) :: avalue(:)

      IF ((axis_hdl%daddr == 0) .OR. &
          .NOT.((axis_hdl%dtype == EAXIS) .OR. (axis_hdl%dtype == GAXIS))) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs à l'axe !"
         STOP
      ELSE
         IF (PRESENT(aname))          THEN
            CALL xios_xml_set_axis_name(axis_hdl%daddr, axis_hdl%dtype, aname, len(aname));
         END IF
         IF (PRESENT(astandard_name)) THEN
            CALL xios_xml_set_axis_sname(axis_hdl%daddr, axis_hdl%dtype, astandard_name, len(astandard_name));
         END IF
         IF (PRESENT(along_name))     THEN
            CALL xios_xml_set_axis_lname(axis_hdl%daddr, axis_hdl%dtype, along_name, len(along_name));
         END IF
         IF (PRESENT(aunit))          THEN
            CALL xios_xml_set_axis_unit(axis_hdl%daddr, axis_hdl%dtype, aunit, len(aunit));
         END IF
         IF (PRESENT(avalue))         THEN
            CALL xios_xml_set_axis_value(axis_hdl%daddr, axis_hdl%dtype, avalue, size(avalue))
         END IF
      END IF
   END SUBROUTINE xml_axis_addAttribut

   SUBROUTINE xml_group_axis_addAttribut(gaxis_hdl, aname, astandard_name, along_name, aunit, avalue)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                          :: gaxis_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN)           :: aname, astandard_name, along_name, aunit
      REAL(kind = 8), dimension(*), OPTIONAL, INTENT(IN) :: avalue(:)

      IF ((gaxis_hdl%daddr == 0) .OR. .NOT.(gaxis_hdl%dtype == GAXIS)) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au groupe d'axes' !"
         STOP
      ELSE
        CALL xml_axis_addAttribut(gaxis_hdl, aname, astandard_name, along_name, aunit, avalue)
      END IF
   END SUBROUTINE xml_group_axis_addAttribut

   SUBROUTINE xml_domain_addAttribut(domain_hdl, dname, dstandard_name, dlong_name,                      &
                                     ni_glo, nj_glo, ibegin, iend, ni, jbegin, jend, nj, mask,           &
                                     data_dim, data_ni, data_nj, data_ibegin, data_jbegin, data_n_index, &
                                     data_i_index, data_j_index, lonvalue_rect, latvalue_rect,           &
                                     lonvalue_curv, latvalue_curv, domtype)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                             :: domain_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN)              :: dname, dstandard_name, dlong_name
      INTEGER, OPTIONAL, INTENT(IN)                         :: ni_glo, nj_glo, ibegin, iend, ni, jbegin, jend, nj
      LOGICAL(kind = 1), DIMENSION(*), OPTIONAL, INTENT(IN) :: mask(:,:)
      INTEGER, OPTIONAL, INTENT(IN)                         :: data_dim, data_ni, data_nj, data_ibegin, data_jbegin, data_n_index
      INTEGER, DIMENSION(*), OPTIONAL, INTENT(IN)           :: data_i_index(:), data_j_index(:)
      REAL(kind = 8), DIMENSION(*), OPTIONAL, INTENT(IN)    :: lonvalue_curv(:,:), latvalue_curv(:,:)
      REAL(kind = 8), DIMENSION(*), OPTIONAL, INTENT(IN)    :: lonvalue_rect(:), latvalue_rect(:)
      CHARACTER(len = *), OPTIONAL, INTENT(IN)              :: domtype

      IF ((domain_hdl%daddr == 0) .OR. &
          .NOT.((domain_hdl%dtype == EDOMAIN) .OR. (domain_hdl%dtype == GDOMAIN))) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au domain !"
         STOP
      ELSE
         IF (PRESENT(dname))              THEN
            CALL xios_xml_set_domain_name(domain_hdl%daddr, domain_hdl%dtype, dname, len(dname));
         END IF
         IF (PRESENT(dstandard_name))     THEN
            CALL xios_xml_set_domain_sname(domain_hdl%daddr, domain_hdl%dtype, dstandard_name, len(dstandard_name));
         END IF
         IF (PRESENT(dlong_name))         THEN
            CALL xios_xml_set_domain_lname(domain_hdl%daddr, domain_hdl%dtype, dlong_name, len(dlong_name));
         END IF
         IF (PRESENT(dlong_name))         THEN
            CALL xios_xml_set_domain_lname(domain_hdl%daddr, domain_hdl%dtype, dlong_name, len(dlong_name));
         END IF
         IF (PRESENT(ni_glo))             THEN
            CALL xios_xml_set_domain_niglo(domain_hdl%daddr, domain_hdl%dtype, ni_glo);
         END IF
         IF (PRESENT(nj_glo))             THEN
            CALL xios_xml_set_domain_njglo(domain_hdl%daddr, domain_hdl%dtype, nj_glo);
         END IF
         IF (PRESENT(ibegin))             THEN
            CALL xios_xml_set_domain_ibegin(domain_hdl%daddr, domain_hdl%dtype, ibegin);
         END IF
         IF (PRESENT(iend))               THEN
            CALL xios_xml_set_domain_iend(domain_hdl%daddr, domain_hdl%dtype, iend);
         END IF
         IF (PRESENT(ni))                 THEN
            CALL xios_xml_set_domain_ni(domain_hdl%daddr, domain_hdl%dtype, ni);
         END IF
         IF (PRESENT(jbegin))             THEN
            CALL xios_xml_set_domain_jbegin(domain_hdl%daddr, domain_hdl%dtype, jbegin);
         END IF
         IF (PRESENT(jend))               THEN
            CALL xios_xml_set_domain_jend(domain_hdl%daddr, domain_hdl%dtype, jend);
         END IF
         IF (PRESENT(nj))                 THEN
            CALL xios_xml_set_domain_nj(domain_hdl%daddr, domain_hdl%dtype, nj);
         END IF
         IF (PRESENT(mask))               THEN
            CALL xios_xml_set_domain_mask(domain_hdl%daddr, domain_hdl%dtype, mask, size(mask, 1), size(mask,2))
         END IF
         IF (PRESENT(data_dim))           THEN
            CALL xios_xml_set_domain_ddim(domain_hdl%daddr, domain_hdl%dtype, data_dim)
         END IF
         IF (PRESENT(data_ni))            THEN
            CALL xios_xml_set_domain_dni(domain_hdl%daddr, domain_hdl%dtype, data_ni)
         END IF
         IF (PRESENT(data_nj))            THEN
            CALL xios_xml_set_domain_dnj(domain_hdl%daddr, domain_hdl%dtype, data_nj)
         END IF
         IF (PRESENT(data_ibegin))        THEN
            CALL xios_xml_set_domain_dibegin(domain_hdl%daddr, domain_hdl%dtype, data_ibegin)
         END IF
         IF (PRESENT(data_jbegin))        THEN
            CALL xios_xml_set_domain_djbegin(domain_hdl%daddr, domain_hdl%dtype, data_jbegin)
         END IF
         IF (PRESENT(data_n_index))       THEN
            CALL xios_xml_set_domain_dnindex(domain_hdl%daddr, domain_hdl%dtype, data_n_index)
         END IF
         IF (PRESENT(data_i_index))       THEN
            CALL xios_xml_set_domain_diindex(domain_hdl%daddr, domain_hdl%dtype, data_i_index, size(data_i_index))
         END IF
         IF (PRESENT(data_j_index))       THEN
            CALL xios_xml_set_domain_djindex(domain_hdl%daddr, domain_hdl%dtype, data_j_index, size(data_j_index))
         END IF
         IF (PRESENT(lonvalue_curv))      THEN
            CALL xios_xml_set_domain_lonvalue(domain_hdl%daddr, domain_hdl%dtype, lonvalue_curv, &
                                              size(lonvalue_curv, 1), size(lonvalue_curv, 2))
         ELSE IF (PRESENT(lonvalue_rect)) THEN
            CALL xios_xml_set_domain_lonvalue(domain_hdl%daddr, domain_hdl%dtype, lonvalue_rect, size(lonvalue_rect, 1), -1)
         END IF
         IF (PRESENT(latvalue_curv))      THEN
            CALL xios_xml_set_domain_latvalue(domain_hdl%daddr, domain_hdl%dtype, latvalue_curv, &
                                              size(latvalue_curv, 1), size(latvalue_curv, 2))
         ELSE IF (PRESENT(latvalue_rect)) THEN
            CALL xios_xml_set_domain_latvalue(domain_hdl%daddr, domain_hdl%dtype, latvalue_rect, size(latvalue_rect, 1), -1)
         END IF
         IF (PRESENT(domtype))            THEN
            CALL xios_xml_set_domain_domtype(domain_hdl%daddr, domain_hdl%dtype, domtype, len(domtype))
         END IF
      END IF
   END SUBROUTINE xml_domain_addAttribut

   SUBROUTINE xml_group_domain_addAttribut(gdomain_hdl, dname, dstandard_name, dlong_name,                     &
                                           ni_glo, nj_glo, ibegin, iend, ni, jbegin, jend, nj, mask,           &
                                           data_dim, data_ni, data_nj, data_ibegin, data_jbegin, data_n_index, &
                                           data_i_index, data_j_index, lonvalue_rect, latvalue_rect,           &
                                           lonvalue_curv, latvalue_curv, domtype)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                             :: gdomain_hdl
      CHARACTER(len = *), OPTIONAL, INTENT(IN)              :: dname, dstandard_name, dlong_name
      INTEGER, OPTIONAL, INTENT(IN)                         :: ni_glo, nj_glo, ibegin, iend, ni, jbegin, jend, nj
      LOGICAL(kind = 1), DIMENSION(*), OPTIONAL, INTENT(IN) :: mask(:,:)
      INTEGER, OPTIONAL, INTENT(IN)                         :: data_dim, data_ni, data_nj, data_ibegin, data_jbegin, data_n_index
      INTEGER, DIMENSION(*), OPTIONAL, INTENT(IN)           :: data_i_index(:), data_j_index(:)
      REAL(kind = 8), DIMENSION(*), OPTIONAL, INTENT(IN)    :: lonvalue_curv(:,:), latvalue_curv(:,:)
      REAL(kind = 8), DIMENSION(*), OPTIONAL, INTENT(IN)    :: lonvalue_rect(:), latvalue_rect(:)
      CHARACTER(len = *), OPTIONAL, INTENT(IN)              :: domtype

      IF ((gdomain_hdl%daddr == 0) .OR. .NOT.(gdomain_hdl%dtype == GDOMAIN)) THEN
         PRINT *, "(F2003 interface) Impossible d'ajouter des attributs au groupe de domaine !"
         STOP
      ELSE
        CALL xml_domain_addAttribut(gdomain_hdl, dname, dstandard_name, dlong_name,                     &
                                    ni_glo, nj_glo, ibegin, iend, ni, jbegin, jend, nj, mask,           &
                                    data_dim, data_ni, data_nj, data_ibegin, data_jbegin, data_n_index, &
                                    data_i_index, data_j_index, lonvalue_rect, latvalue_rect,           &
                                    lonvalue_curv, latvalue_curv, domtype)
      END IF
   END SUBROUTINE xml_group_domain_addAttribut

END MODULE IXMLTREE

! --------------------------------------------- !
!                     IXML                      !
!       Prise en charge des document XML        !
! --------------------------------------------- !
MODULE IXML
   USE ISO_C_BINDING

   INTERFACE ! Ne pas appeler directement/Interface FORTRAN 2003 <-> C99

      SUBROUTINE xios_xml_Parse_File(filename, filename_size) bind(C)
         import C_CHAR, C_INT
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: filename
         INTEGER  (kind = C_INT), VALUE         :: filename_size
      END SUBROUTINE xios_xml_Parse_File

      SUBROUTINE xios_xml_Parse_String(xmlcontent, xmlcontent_size) bind(C)
         import C_CHAR, C_INT
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: xmlcontent
         INTEGER  (kind = C_INT), VALUE         :: xmlcontent_size
      END SUBROUTINE xios_xml_Parse_String

   END INTERFACE

   CONTAINS ! Fonctions disponibles pour les utilisateurs.

   ! Parsing d'un document XML depuis un fichier dont le chemin est transmis en argument.
   SUBROUTINE xml_Parse_File(filename)
      CHARACTER(len = *), INTENT(IN) :: filename
      CALL xios_xml_Parse_File(filename, len(filename))
   END SUBROUTINE xml_Parse_File

   ! Parsing d'un document XML depuis une chaîne de caractère.
   SUBROUTINE xml_Parse_String(xmlcontent)
      CHARACTER(len = *), INTENT(IN) :: xmlcontent
      CALL xios_xml_Parse_String(xmlcontent, len(xmlcontent))
   END SUBROUTINE xml_Parse_String

END MODULE IXML

! ------------------------------------------------ !
!                   ICONTEXT                       !
! Ajout, suppression, ... de contextes à la config !
! ------------------------------------------------ !
MODULE ICONTEXT
   USE ISO_C_BINDING

   INTERFACE ! Ne pas appeler directement/Interface FORTRAN 2003 <-> C99

      SUBROUTINE xios_context_set_current(context, withswap) bind(C)
         import C_BOOL, C_INT
         INTEGER  (kind = C_INT), VALUE :: context
         LOGICAL (kind = C_BOOL), VALUE :: withswap
      END SUBROUTINE xios_context_set_current

      SUBROUTINE xios_context_create(context, context_id, context_id_size, calendar_type) bind(C)
         import C_CHAR, C_INT
         INTEGER  (kind = C_INT)                :: context
         CHARACTER(kind = C_CHAR), DIMENSION(*) :: context_id
         INTEGER  (kind = C_INT), VALUE         :: context_id_size
         INTEGER  (kind = C_INT), VALUE         :: calendar_type
      END SUBROUTINE xios_context_create

   END INTERFACE

   CONTAINS ! Fonctions disponibles pour les utilisateurs.

   ! Permet de changer de contexte de travail.
   SUBROUTINE context_set_current(context, withswap)
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN)                :: context
      LOGICAL (kind = 1), OPTIONAL, INTENT(IN) :: withswap
      LOGICAL (kind = 1)                       :: wswap
      IF (PRESENT(withswap)) THEN
         wswap = withswap
      ELSE
         wswap = .FALSE.
      END IF
      IF ((context%dtype .NE. ECONTEXT) .OR. (context%daddr .EQ. 0)) THEN
         PRINT *, "(F2003 interface) Impossible de se placer dans le nouveau context car le handle est invalide !"
         STOP
      END IF
      CALL xios_context_set_current(context%daddr, wswap)
   END SUBROUTINE context_set_current

   ! Créer un nouveau contexte à partir d'un id, d'un type de calendrier et d'une date.
   SUBROUTINE context_create(context_hdl, context_id, calendar_type)
      USE IXHANDLE
      TYPE(XHandle), INTENT(OUT)     :: context_hdl
      CHARACTER(len = *), INTENT(IN) :: context_id
      INTEGER, INTENT(IN)            :: calendar_type
      CALL xios_context_create(context_hdl%daddr, context_id, len(context_id), calendar_type)
      IF (context_hdl%daddr .EQ. 0) THEN
         PRINT *, "(F2003 interface) Impossible de créer le context !"
         STOP
      ELSE
         context_hdl%dtype = ECONTEXT
      END IF
   END SUBROUTINE context_create

END MODULE ICONTEXT

! --------------------------------------------- !
!                   IDATATREATMENT              !
!   Traitement des données issues des codes.    !
! --------------------------------------------- !

MODULE IDATATREATMENT
   USE ISO_C_BINDING

   ! enum XFileType
   INTEGER, PARAMETER  :: NETCDF4 = 0

   INTERFACE ! Ne pas appeler directement/Interface FORTRAN 2003 <-> C99

      SUBROUTINE xios_dtreatment_start(context_hdl, filetype) bind(C)
         import C_INT
         INTEGER  (kind = C_INT), VALUE  :: context_hdl
         INTEGER  (kind = C_INT), VALUE  :: filetype
      END SUBROUTINE xios_dtreatment_start

   END INTERFACE

   CONTAINS ! Fonctions disponibles pour les utilisateurs.

   SUBROUTINE dtreatment_start(context_hdl, filetype)
      USE ICONTEXT
      USE IXHANDLE
      TYPE(XHandle), INTENT(IN), VALUE :: context_hdl
      INTEGER, INTENT(IN), OPTIONAL    :: filetype
      INTEGER                          :: filetype_
      IF ((context_hdl%dtype .NE. ECONTEXT) .OR. (context_hdl%daddr .EQ. 0)) THEN
         PRINT *, "(F2003 interface) Impossible de traiter un contexte invalide !"
         STOP
      END IF
      IF (PRESENT(filetype)) THEN
         filetype_ = filetype
      ELSE
         filetype_ = NETCDF4
      END IF
      CALL context_set_current(context_hdl)
      CALL xios_dtreatment_start(context_hdl%daddr, filetype_)
   END SUBROUTINE dtreatment_start

END MODULE IDATATREATMENT

! --------------------------------------------- !
!                   IEXTENDED                   !
!        Quelques fonctions pratiques  ...      !
! --------------------------------------------- !

MODULE IEXTENDED
   USE ISO_C_BINDING



END MODULE IEXTENDED

