#ifndef __C_INTERFACE__
#define __C_INTERFACE__

#include "xmlio.hpp"

using namespace XMLIOSERVER;
using namespace XMLIOSERVER::XML;

typedef long int XInt, XSize; // Integer
typedef bool     XBool;       // Logical
typedef void   * XPtr;        // Pointeur C non typé
typedef char   * XString;     // Pointeur C non typé

typedef enum
{
   NOTYPE = 0,
   DTREATMENT/* UNUSED */, DATE, CALENDAR, ECONTEXT,
   EAXIS, EDOMAIN, EFIELD, EFILE, EGRID,
   GAXIS, GDOMAIN, GFIELD, GFILE, GGRID
} XDType;

typedef enum
{
   NETCDF4 = 0
} XFileType;

typedef enum
 { D360 = 0 , ALLLEAP, NOLEAP, JULIAN, GREGORIAN } XCalendarType ;

typedef struct // Référence C à une objet de la bibliothèque.
{ XDType data_type; XPtr data_ptr; } XHandle;

#define MAKE_STRING(var_name, str_arg, int_arg)             \
      std::string var_name##temp;                           \
      var_name##temp.append (str_arg, int_arg);             \
      size_t d = var_name##temp.find_first_not_of(' ');     \
      size_t f = var_name##temp.find_last_not_of (' ');     \
      std::string var_name = var_name##temp.substr(d, f-d+1)

#define XML_SET(type, type_var, var, long_var) \
   void xios_xml_set_##type##_##var(XPtr const _##type, XDType _dtype, type_var var)

#define XML_SET_STRING(type, var, long_var) \
   void xios_xml_set_##type##_##var(XPtr const _##type, XDType _dtype, const XString _##var, XSize _##var##_len)

#ifdef __cplusplus
extern"C"
{
#endif //__cplusplus

/* ******************** HANDLE INTERFACE ******************** */
void xios_handle_create (XPtr * const _ret, XDType _dtype, const XString _id, XSize _id_len);

/* ******************** XML INTERFACE *********************** */
void xios_xml_parse_file  (const XString _filename, XSize _filename_len);
void xios_xml_parse_string(const XString _xmlcontent, XSize _xmlcontent_len);

/* ******************** CONTEXT INTERFACE ******************* */
void xios_context_set_current (XPtr const _ctx, bool _wswap);
void xios_context_create (XPtr * _ctx, const XString _ctx_id, XSize _ctx_id_len, XCalendarType _calType);

/* ******************  XML TREE INTERFACE ******************* */
void xios_xml_tree_add(const XPtr _parent, XDType _parent_type, XPtr * _child,
                       XDType _child_type, const XString _child_id, XSize _child_id_len);
void xios_xml_tree_show(const XString _filename, XSize _filename_len);

// Attributs de field ou field_group
XML_SET_STRING(field, name, name);
XML_SET_STRING(field, sname, standard_name);
XML_SET_STRING(field, lname, long_name);
XML_SET_STRING(field, unit, unit);
XML_SET_STRING(field, operation, operation);

void xios_xml_set_field_freq_op(XPtr const _field, XDType _dtype,
                                 double year, double month, double day,
                                 double hour, double minute, double second);

XML_SET(field, int, level, level);
XML_SET(field, int, prec, prec);
XML_SET(field, bool, enabled, enabled);

XML_SET_STRING(field, dref, domain_ref);
XML_SET_STRING(field, aref, axis_ref);
XML_SET_STRING(field, gref, grid_ref);
XML_SET_STRING(field, zref, zoom_ref);
XML_SET_STRING(field, fref, field_ref);

// Attributs de file ou file_group
XML_SET_STRING(file, name, name) ;
XML_SET_STRING(file, description, description);
void xios_xml_set_file_output_freq(XPtr const _file, XDType _dtype,
                                   double year, double month, double day,
                                   double hour, double minute, double second);

XML_SET(file, int, olevel, output_level);
XML_SET(file, bool, enabled, enabled);

// Attributs de grid ou grid_group
XML_SET_STRING(grid, name, name) ;
XML_SET_STRING(grid, description, description) ;
XML_SET_STRING(grid, dref, domain_ref) ;
XML_SET_STRING(grid, aref, axis_ref) ;

// Attribut de axis ou axis_group
XML_SET_STRING(axis, name, name) ;
XML_SET_STRING(axis, sname, standard_name) ;
XML_SET_STRING(axis, lname, long_name) ;

XML_SET_STRING(axis, unit, unit) ;

void xios_xml_set_axis_value(XPtr const _file, XDType _dtype, double value[], XSize value_size);


// Attribut de domain ou domain_group
XML_SET_STRING(domain, name, name) ;
XML_SET_STRING(domain, sname, standard_name) ;
XML_SET_STRING(domain, lname, long_name) ;

XML_SET(domain, int, niglo, ni_glo);
XML_SET(domain, int, njglo, nj_glo);

XML_SET(domain, int, ibegin, ibegin);
XML_SET(domain, int, iend, iend);
XML_SET(domain, int, ni, ni);

XML_SET(domain, int, jbegin, jbegin);
XML_SET(domain, int, jend, jend);
XML_SET(domain, int, nj, nj);

void xios_xml_set_domain_mask(XPtr const _domain, XDType _dtype, bool * _mask , XSize _maskXsize, XSize _maskYsize);

XML_SET(domain, int, ddim, data_dim);
XML_SET(domain, int, dni, data_ni);
XML_SET(domain, int, dnj, data_nj);
XML_SET(domain, int, dibegin, data_ibegin);
XML_SET(domain, int, djbegin, data_jbegin);

XML_SET(domain, int, dnindex, data_n_index);

void xios_xml_set_domain_diindex(XPtr const _domain, XDType _dtype, int _diindex[], XSize _diindex_size);
void xios_xml_set_domain_djindex(XPtr const _domain, XDType _dtype, int _djindex[], XSize _djindex_size);

void xios_xml_set_domain_lonvalue(XPtr const _domain, XDType _dtype, double _lonvalue[], XSize _lonvalue_Xsize, XSize _lonvalue_Ysize);
void xios_xml_set_domain_latvalue(XPtr const _domain, XDType _dtype, double _latvalue[], XSize _latvalue_Xsize, XSize _latvalue_Ysize);

XML_SET_STRING(domain, domtype, domtype) ;

/* ******************  DATA TREATMENT INTERFACE ******************* */
void xios_dtreatment_start(XPtr const _context, XFileType filetype);

#ifdef __cplusplus
}
#endif //__cplusplus


#endif //__C_INTERFACE__
