#ifndef __DOMAIN__
#define __DOMAIN__

using XMLIOSERVER::XML::XMLNode;
using XMLIOSERVER::XML::THashAttributes;

namespace XMLIOSERVER
{
   class CDomain : public ObjectTemplate<CDomain>, public DomainAttribut
   {
      public:

         CDomain(void)
            : ObjectTemplate<CDomain>(), DomainAttribut()
         { /* Ne rien faire de plus */ }

         CDomain(const string& _id)
            : ObjectTemplate<CDomain>(_id), DomainAttribut()
         { /* Ne rien faire de plus */ }

         static string GetName(void)    { return ("domain"); }
         static string GetDefName(void) { return (CDomain::GetName()); }

         inline void check(void) ;

         ~CDomain(void)
         { /* Ne rien faire de plus */ }

   }; // class Domain

   void CDomain::check(void)
   { // Recodée en partie le 17/11/10

      // Domaine global //
      if ((!ni_glo.hasValue() || ni_glo <= 0 ) ||
          (!nj_glo.hasValue() || nj_glo <= 0 ))
        ERROR("Le domaine global est mal défini, vérifiez les valeurs de \'ni_glo\' et \'nj_glo\' !") ;

      // Domaine local en i //
      if (ni.hasValue() && ibegin.hasValue() && !iend.hasValue())
         iend = ibegin + ni - 1 ;
      else if (ni.hasValue() && iend.hasValue() && !ibegin.hasValue())
         ibegin = ni - iend + 1 ;
      else if (ibegin.hasValue() && iend.hasValue() && !ni.hasValue())
         ni = iend - ibegin - 1 ;
      else if (ibegin.hasValue() && iend.hasValue() &&
               ni.hasValue() && (iend != ibegin + ni - 1))
         ERROR("Le domaine est mal défini, iend est différent de (ibegin + ni - 1) !") ;
      else
         ERROR("Le domaine est mal défini, deux valeurs au moins parmis iend, ibegin, ni doivent être définies !") ;

      if (ni < 0 || ibegin > iend ||
          ibegin < 1 || iend > ni_glo)
         ERROR("Domaine local mal défini, vérifiez les valeurs ni, ni_glo, ibegin, iend") ;

      // Domaine local en j //
      if (nj.hasValue() && jbegin.hasValue() && !jend.hasValue())
         jend = jbegin + nj - 1 ;
      else if (nj.hasValue() && jend.hasValue() && !jbegin.hasValue())
         jbegin = nj - jend + 1 ;
      else if (jbegin.hasValue() && jend.hasValue() && !nj.hasValue())
         nj = jend - jbegin - 1 ;
      else if (jbegin.hasValue() && jend.hasValue() &&
               nj.hasValue() && (jend != jbegin + nj - 1))
         ERROR("Le domaine est mal défini, jend est différent de (jbegin + nj - 1) !") ;
      else
         ERROR("Le domaine est mal défini, deux valeurs au moins parmis jend, jbegin, nj doivent être définjes !") ;

      if (nj < 0 || jbegin > jend ||
          jbegin < 1 || jend > nj_glo)
         ERROR("Domaine local mal défini, vérifiez les valeurs nj, nj_glo, jbegin, jend") ;

      // Masque de domaine //
      if (mask.hasValue())
      {
         if ((mask->extent(0) != ni) ||
             (mask->extent(1) != nj))
            ERROR("Le masque n'a pas la même taille que le domaine local") ;
      }
      else // (!mask.hasValue())
      { // Si aucun masque n'est défini, on en crée un nouveau qui valide l'intégralité du domaine.
         Array<bool, 2> __arr(shape(ni, nj));
         __arr = true ;
         mask->resize(shape(ni,nj)) ;
         this->set_mask(__arr);
      }

      // Données du domaine //
      if (!data_jbegin.hasValue()) data_jbegin = 0 ;
      if (!data_ibegin.hasValue()) data_ibegin = 0 ;

      if (data_dim.hasValue() &&
        !(data_dim==1 || data_dim==2))
         ERROR("Dimension des données non comptatible (doit être 1 ou 2) !") ;
      else if (! data_dim.hasValue())
         ERROR("Dimension des données non définie !") ;

      if (data_ni.hasValue() && (data_ni <= 0))
         ERROR("Dimension des données négative (data_ni).") ;
      else if (!data_ni.hasValue())
         data_ni = (data_dim == 1) ? (ni * nj) : (int) ni;

      if (data_nj.hasValue() && (data_nj <= 0))
         ERROR("Dimension des données négative (data_nj).") ;
      else if (!data_nj.hasValue())
         data_nj = (int) nj ;

      // Gestion de la compression //
      if (data_i_index.hasValue())
      {
         if (data_n_index.hasValue() &&
            (data_n_index != data_i_index->extent(0)))
            ERROR("Dimension data_i_index incompatible avec data_n_index.") ;
         else if (!data_n_index.hasValue())
            data_n_index = data_i_index->extent(0) ;

         if (data_dim == 2)
         {
            if (data_j_index.hasValue() &&
               (data_j_index->extent(0) != data_i_index->extent(0)))
               ERROR("Dimension data_j_index incompatible avec data_i_index.") ;
            else if (!data_j_index.hasValue())
               ERROR("La donnée data_j_index doit être renseignée !") ;
         }
      }
      else
      {
         if (data_n_index.hasValue() ||
            ((data_dim == 2) && (data_j_index.hasValue())))
            ERROR("data_i_index non défini") ;
      }

      if (!data_n_index.hasValue())
      { // -> bloc re-vérfié OK
         if (data_dim == 1)
         {
            const int dni = data_ni;
            Array<int, 1> __arri(shape(dni));
            this->set_data_n_index(dni) ;
            for (int i = 0; i < dni; i++)
               __arri(i) = i+1 ;
            data_i_index->resize(dni) ;
            this->set_data_i_index(__arri) ;
         }
         else   // (data_dim == 2)
         {
            const int dni = data_ni * data_nj;
            Array<int, 1> __arri(shape(dni)), __arrj(shape(dni));
            this->set_data_n_index(dni) ;
            for(int count = 0, i = 0; i  <data_ni; i++)
               for(int j = 0; j < data_nj; j++, count++)
               { __arri(count) = i+1 ;  __arrj(count) = j+1 ; }
            data_i_index -> resize(dni) ;
            data_j_index -> resize(dni) ;
            this->set_data_i_index(__arri) ;
            this->set_data_j_index(__arrj) ;
         }
      }
   }

} // namespace XMLIOSERVER

DECLARE_GROUP(Domain)

#endif // __FIELD__
