! --------------------------------------------------- !
!               XMLIO SERVER MAIN TEST                !
! --------------------------------------------------- !

PROGRAM MAIN
   USE IXHANDLE
   USE IXML
   USE IXMLTREE
   USE ICONTEXT
   USE ICALENDAR
   USE IEXTENDED
   USE IDATATREATMENT

   INTEGER       :: compt = 0

   TYPE(XHandle) :: nemo_style_ctxt     = NULLHANDLE, &
                    orchidee_style_ctxt = NULLHANDLE, &
                    lmdz_style_ctxt     = NULLHANDLE

   TYPE(XHandle) :: temp_mod  = NULLHANDLE, &
                    temp_mod_ = NULLHANDLE

   ! Axes verticaux
   REAL(kind = 8), DIMENSION(10) :: vaxis  = (/(i, i=1, 10)/)
                                    !vvaxis = (/(i, i=1, 5)/)

   ! Domaines horizontaux
   REAL(kind = 8), DIMENSION(20) :: lonrect_orch0  = (/(i, i=11, 30)/),&
                                    latrect_orch0  = (/(i, i=11, 30)/)

   REAL(kind = 8), DIMENSION(10,10) :: loncurv_nemo0  = 1, &
                                       latcurv_nemo0  = 1

   ! Les durées
   TYPE(XDuration) :: timestep_1h = XDuration(0., 0., 0., 1., 0., 0.)
                      !timestep_2h = XDuration(0., 0., 0., 2., 0., 0.)

   ! Les masques
   LOGICAL(kind = 1), DIMENSION(20, 20) :: mask0  = .TRUE._1

   CHARACTER(len = *), PARAMETER :: filename = &
            "/local/XMLIOSERVER_DEV/dev_rv/test/iodef_simple_test.xml"

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   ! Parsing du document xml depuis un fichier situé sur le disque.
   CALL xml_Parse_File(filename)

   ! Création de handle sur les contextes exstants dans lequels on souhaite travailler.
   CALL handle_create(nemo_style_ctxt, ECONTEXT, "nemo_style")
   CALL context_set_current(nemo_style_ctxt)

   CALL handle_create(temp_mod, EDOMAIN,  "domaine0")
      CALL xml_domain_addAttribut(domain_hdl    = temp_mod,      &
                                  lonvalue_curv = loncurv_nemo0, &
                                  latvalue_curv = latcurv_nemo0, &
                                  domtype       = "curvilinear")

   CALL handle_create(lmdz_style_ctxt, ECONTEXT, "lmdz_style")
   CALL context_set_current(lmdz_style_ctxt)

   !!!!!!!!!!!!!!!!!!!! EXEMPLE RECONSTRUCTION !!!!!!!!!!!!!!!!!!!!!
   ! On crée un nouveau context et on lui associe un handle.
   CALL context_create(context_hdl   = orchidee_style_ctxt, &
                       context_id    = "orchidee_style",    &
                       calendar_type = GREGORIAN)

   ! ---------> field_définition
   CALL handle_create(temp_mod, GFIELD,  "field_definition")
      CALL xml_tree_add(parent_hdl = temp_mod,        &
                        child_hdl  = temp_mod_,       &
                        child_type = GFIELD,          &
                        child_id   = "ss_fieldgroup")
      CALL xml_group_field_addAttribut(gfield_hdl = temp_mod_, &
                                       flevel     = 1,         &
                                       fprec      = 4,         &
                                       fenabled   = .TRUE._1)
         CALL xml_tree_add(parent_hdl = temp_mod_,    &
                           child_hdl  = temp_mod ,    &
                           child_type = EFIELD,       &
                           child_id   = "field0")
         CALL xml_field_addAttribut(field_hdl      = temp_mod,       &
                                    fname          = "field0_name",  &
                                    fstandard_name = "field0_sname", &
                                    flong_name     = "field0_lname", &
                                    funit          = "Pa",           &
                                    foperation     = "inst",         &
                                    ffreq_op       = timestep_1h,    &
                                    fgrid_ref      = "grid0")
   ! ---------> axis_définition
   CALL handle_create(temp_mod, GAXIS,   "axis_definition")
      CALL xml_tree_add(parent_hdl = temp_mod,        &
                        child_hdl  = temp_mod_,       &
                        child_type = GAXIS,           &
                        child_id   = "ss_axisgroup")
      CALL xml_group_axis_addAttribut(gaxis_hdl = temp_mod_, &
                                      aunit     = "SI")
         CALL xml_tree_add(parent_hdl = temp_mod_,    &
                           child_hdl  = temp_mod ,    &
                           child_type = EAXIS,        &
                           child_id   = "axis0")
         CALL xml_axis_addAttribut(axis_hdl       = temp_mod,             &
                                   aname          = "saxev",              &
                                   astandard_name = "saxev",              &
                                   along_name     = "Simple axe vertical",&
                                   avalue         = vaxis)

   ! ---------> domain_définition
   CALL handle_create(temp_mod, GDOMAIN, "domain_definition")
      CALL xml_tree_add(parent_hdl = temp_mod,        &
                        child_hdl  = temp_mod_,       &
                        child_type = GDOMAIN,         &
                        child_id   = "ss_domaingroup")
      CALL xml_group_domain_addAttribut(gdomain_hdl = temp_mod_, &
                                        ni_glo      = 40,        &
                                        nj_glo      = 40)
         CALL xml_tree_add(parent_hdl = temp_mod_,    &
                           child_hdl  = temp_mod ,    &
                           child_type = EDOMAIN,      &
                           child_id   = "domain0")
         CALL xml_domain_addAttribut(domain_hdl     = temp_mod,             &
                                     dname          = "Compressee",         &
                                     dstandard_name = "DomOrchTest",        &
                                     dlong_name     = "Test type orchidee", &
                                     ibegin         = 10,                   &
                                     iend           = 31,                   &
                                     jbegin         = 10,                   &
                                     jend           = 31,                   &
                                     mask           = mask0,                &
                                     data_dim       = 1,                    &
                                     data_ni        = 400,                  &
                                     data_ibegin    = 1,                    &
                                     lonvalue_rect  = lonrect_orch0,        &
                                     latvalue_rect  = latrect_orch0,        &
                                     domtype        = "rectilinear")

   ! ---------> file_définition
   CALL handle_create(temp_mod, GFILE,   "file_definition")
      CALL xml_tree_add(parent_hdl = temp_mod,        &
                        child_hdl  = temp_mod_,       &
                        child_type = GFILE)
      CALL xml_group_file_addAttribut(gfile_hdl = temp_mod_,   &
                                      fofreq   = timestep_1h,  &
                                      folevel   = 2,           &
                                      fenabled  = .TRUE._1)
         CALL xml_tree_add(parent_hdl = temp_mod_,    &
                           child_hdl  = temp_mod ,    &
                           child_type = EFILE,        &
                           child_id   = "file0")
         CALL xml_file_addAttribut(file_hdl     = temp_mod,                      &
                                   fname        = "data/orchidee_test.nc",       &
                                   fdescription = "Test sur grille type orchidee")
            CALL xml_tree_add(parent_hdl = temp_mod , &
                              child_hdl  = temp_mod_, &
                              child_type = EFIELD)
            CALL xml_field_addAttribut(field_hdl  = temp_mod_, &
                                       ffield_ref = "field0")

   ! ---------> grid_définition
   CALL handle_create(temp_mod, GGRID,   "grid_definition")
      CALL xml_tree_add(parent_hdl = temp_mod,        &
                        child_hdl  = temp_mod_,       &
                        child_type = GGRID,           &
                        child_id   = "ss_gridgroup")
      CALL xml_group_grid_addAttribut(ggrid_hdl    = temp_mod_, &
                                      gdomain_ref  = "domain0", &
                                      gaxis_ref    = "axis0");
         CALL xml_tree_add(parent_hdl = temp_mod_,    &
                           child_hdl  = temp_mod ,    &
                           child_type = EGRID,        &
                           child_id   = "grid0")
         CALL xml_grid_addAttribut(grid_hdl     = temp_mod,           &
                                   gname        = "GRectiCompressee", &
                                   gdescription = "Grille orchidee");

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   ! Affichage de l'arbre xml au niveau de la sortie.
   CALL xml_tree_show("xmlio_tree_prev.log")
   ! CALL xml_tree_show()

   ! ----------- Début du traitement ----------- !
   ! On choisit le context dans lequel on va travailler et on commence le traitement des données.
   CALL dtreatment_start(orchidee_style_ctxt, NETCDF4)

   ! Exécution de la boucle de calcul-écriture.
   DO compt = 0, 9

   END DO

   ! Affichage de l'arbre xml au niveau de la sortie.
   CALL xml_tree_show("xmlio_tree_next.log")

END PROGRAM MAIN
